use based::asset::AssetRoutes;
use based::auth::User;
use based::get_pg;
use based::ui::components::Shell;
use based::ui::prelude::*;
use config::Config;
use rocket::routes;

pub mod config;
pub mod routes;

#[rocket::launch]
async fn launch() -> _ {
    env_logger::init();

    let pg = get_pg!();
    sqlx::migrate!("./migrations").run(pg).await.unwrap();

    let config: Config =
        toml::from_str(&std::fs::read_to_string("config.toml").unwrap_or_default())
            .unwrap_or_default();

    let _ = User::create("admin".to_string(), "admin", based::auth::UserRole::Admin).await;

    let shell = Shell::new(
        Nothing(),
        Nothing(),
        Background(MinHeight(
            ScreenValue::screen,
            Padding(Text("").color(&Gray::_200)).all(ScreenValue::_10),
        ))
        .color(Gray::_900),
    )
    .use_ui();

    rocket::build()
        .mount_assets()
        .mount("/", routes![
            routes::index_page,
            routes::pkg_route,
            routes::push::upload_pkg,
            routes::user::login,
            routes::user::login_post,
            routes::user::account_page,
            routes::ui::pkg_ui,
            routes::ui::repo_ui,
            routes::user::new_api_key,
            routes::user::end_session,
            routes::user::change_password,
            routes::user::change_password_post
        ])
        .manage(config)
        .manage(shell)
}
